import 'dart:math';
import 'package:meta/meta.dart';
import 'package:collection/collection.dart';

part 'iterable_value.dart';
part 'list_value.dart';
part 'set_value.dart';
part 'map_value.dart';

@immutable
abstract class _CollectionValue<T> {
  final T _base;
  final _CollectionEquality<T> _equality;

  _CollectionValue(this._base, this._equality) {
    validate();
  }

  void validate() {}

  @override
  String toString() => _base.toString();
}

class _CollectionEquality<T> {
  final Equality<T> _equality;
  int _cachedHashCode;

  _CollectionEquality(this._equality);

  bool equals(T e1, T e2) =>
      e1.hashCode == e2.hashCode && _equality.equals(e1, e2);

  int getHashCode(T e) => _cachedHashCode ??= _equality.hash(e);
}

class NullError extends Error {
  @override
  String toString() => 'Cannot contain null';
}

@alwaysThrows
R _throwNullError<R>() {
  throw NullError();
}
