part of 'collection_value.dart';

class ListValue<E> extends _CollectionValue<List<E>>
    with
        _ImmutableIterableMixin<E, List<E>>,
        _ImmutableListMixin<E>,
        _MutableWithThrowListMixin<E> {
  ListValue(Iterable<E> value)
      : super(
          value?.any((e) => e == null) ?? true
              ? _throwNullError()
              : List.of(value),
          _CollectionEquality(ListEquality<E>()),
        );

  @override
  int get hashCode => _equality.getHashCode(this);

  @override
  bool operator ==(Object other) =>
      other is ListValue<E> && _equality.equals(this, other);
}

mixin _ImmutableListMixin<E> on _ImmutableIterableMixin<E, List<E>>
    implements List<E> {
  @override
  List<E> operator +(covariant List<E> other) => _base + other;

  @override
  E operator [](int index) => _base[index];

  @override
  Map<int, E> asMap() => _base.asMap();

  @override
  List<R> cast<R>() => _base.cast();

  @override
  Iterable<E> getRange(int start, int end) => _base.getRange(start, end);

  @override
  int indexOf(E element, [int start = 0]) => _base.indexOf(element, start);

  @override
  int indexWhere(bool Function(E element) test, [int start = 0]) =>
      _base.indexWhere(test, start);

  @override
  int lastIndexOf(E element, [int start]) => _base.lastIndexOf(element, start);

  @override
  int lastIndexWhere(bool Function(E element) test, [int start]) =>
      _base.lastIndexWhere(test, start);

  @override
  Iterable<E> get reversed => _base.reversed;

  @override
  List<E> sublist(int start, [int end]) => _base.sublist(start, end);
}

mixin _MutableWithThrowListMixin<E> on _ImmutableListMixin<E> {
  @alwaysThrows
  R _throw<R>() {
    throw UnsupportedError('Cannot modify an immutable List<$E>');
  }

  @override
  set first(E value) => _throw();

  @override
  set last(E value) => _throw();

  @override
  set length(int newLength) => _throw();

  @override
  void operator []=(int index, E value) => _throw();

  @override
  void add(E value) => _throw();

  @override
  void addAll(Iterable<E> iterable) => _throw();

  @override
  void clear() => _throw();

  @override
  void fillRange(int start, int end, [E fillValue]) => _throw();

  @override
  void insert(int index, E element) => _throw();

  @override
  void insertAll(int index, Iterable<E> iterable) => _throw();

  @override
  bool remove(Object value) => _throw();

  @override
  E removeAt(int index) => _throw();

  @override
  E removeLast() => _throw();

  @override
  void removeRange(int start, int end) => _throw();

  @override
  void removeWhere(bool Function(E element) test) => _throw();

  @override
  void replaceRange(int start, int end, Iterable<E> replacement) => _throw();

  @override
  void retainWhere(bool Function(E element) test) => _throw();

  @override
  void setAll(int index, Iterable<E> iterable) => _throw();

  @override
  void setRange(int start, int end, Iterable<E> iterable,
          [int skipCount = 0]) =>
      _throw();

  @override
  void shuffle([Random random]) => _throw();

  @override
  void sort([int Function(E a, E b) compare]) => _throw();
}
