part of 'collection_value.dart';

class MapValue<K, V> extends _CollectionValue<Map<K, V>>
    with _ImmutableMapMixin<K, V>, _MutableWithThrowMapMixin<K, V> {
  MapValue(Map<K, V> value)
      : super(
          value?.entries?.any(
                      (e) => e == null || e.key == null || e.value == null) ??
                  true
              ? _throwNullError()
              : Map.of(value),
          _CollectionEquality(MapEquality<K, V>()),
        );

  @override
  int get hashCode => _equality.getHashCode(this);

  @override
  bool operator ==(Object other) =>
      other is MapValue<K, V> && _equality.equals(this, other);
}

mixin _ImmutableMapMixin<K, V> on _CollectionValue<Map<K, V>>
    implements Map<K, V> {
  @override
  V operator [](Object key) => _base[key];

  @override
  Map<RK, RV> cast<RK, RV>() => _base.cast();

  @override
  bool containsKey(Object key) => _base.containsKey(key);

  @override
  bool containsValue(Object value) => containsValue(value);

  @override
  Iterable<MapEntry<K, V>> get entries => _base.entries;

  @override
  void forEach(void Function(K key, V value) f) => _base.forEach(f);

  @override
  bool get isEmpty => _base.isEmpty;

  @override
  bool get isNotEmpty => _base.isNotEmpty;

  @override
  Iterable<K> get keys => _base.keys;

  @override
  int get length => _base.length;

  @override
  Map<K2, V2> map<K2, V2>(MapEntry<K2, V2> Function(K key, V value) f) =>
      _base.map(f);

  @override
  Iterable<V> get values => _base.values;
}

mixin _MutableWithThrowMapMixin<K, V> on _ImmutableMapMixin<K, V> {
  @alwaysThrows
  R _throw<R>() {
    throw UnsupportedError('Cannot modify an immutable Map<$K, $V>');
  }

  @override
  void operator []=(K key, V value) => _throw();

  @override
  void addAll(Map<K, V> other) => _throw();

  @override
  void addEntries(Iterable<MapEntry<K, V>> newEntries) => _throw();

  @override
  void clear() => _throw();

  @override
  V putIfAbsent(K key, V Function() ifAbsent) => _throw();

  @override
  V remove(Object key) => _throw();

  @override
  void removeWhere(bool Function(K key, V value) predicate) => _throw();

  @override
  V update(K key, V Function(V value) update, {V Function() ifAbsent}) =>
      _throw();

  @override
  void updateAll(V Function(K key, V value) update) => _throw();
}
